#!<%= ENV["SHELL"] || "/bin/bash" %>

<%- if tmux_has_session? name -%>
  cd <%= root || "." %>

  # Run on_project_stop command
  <%= hook_on_project_stop %>

  <%= tmux_kill_session_command %>
<%- end -%>
